import stactools.core

stactools.core.use_fsspec()


def register_plugin(registry):
    # Register subcommands

    from stactools.landsat import commands

    registry.register_subcommand(commands.create_landsat_command)


__version__ = '0.2.1'



import os

import click
from click import Command, Group
from pystac import CatalogType, Item
from stactools.core.utils.antimeridian import Strategy

from stactools.landsat.stac import create_collection, create_stac_item
from stactools.landsat.utils import transform_stac_to_stac


def create_landsat_command(cli: Group) -> Command:
    """Creates a command group for commands working with the USGS's Landsat
    Collection 2 metadata.
    """

    @cli.group(
        'landsat',
        short_help=("Commands for working with Landsat Collection 2 metadata.")
    )
    def landsat() -> None:
        pass

    @landsat.command(
        "create-item",
        short_help=(
            "Creates a STAC Item from Landsat Collection 2 scene metadata."))
    @click.option("-m",
                  "--mtl",
                  required=True,
                  help="HREF to the source MTL metadata xml file.")
    @click.option("-o",
                  "--output",
                  required=True,
                  help="HREF of directory in which to write the item.")
    @click.option("-u",
                  "--usgs_geometry",
                  is_flag=True,
                  help="Use USGS STAC Item geometry")
    @click.option("-l",
                  "--legacy_l8",
                  is_flag=True,
                  help="Create legacy Landsat 8 STAC Item")
    @click.option("-a",
                  "--antimeridian_strategy",
                  type=click.Choice(["normalize", "split"],
                                    case_sensitive=False),
                  default="split",
                  show_default=True,
                  help="geometry strategy for antimeridian scenes")
    @click.option("--level",
                  type=click.Choice(['level-1', 'level-2'],
                                    case_sensitive=False),
                  default="level-2",
                  show_default=True,
                  help="Product level to process. Unused.")
    def create_item_cmd(level: str, mtl: str, output: str, usgs_geometry: bool,
                        legacy_l8: bool, antimeridian_strategy: str) -> None:
        """\b
        Creates a STAC Item for a Landsat Collection 2 scene based on metadata
        from a Landsat MTL xml file.

        \b
        The following Landsat Collection 2 processing Levels and sensors are
        supported:
            Level-1
                - Landsat 1-5 Multi Spectral Scanner (MSS)
            Level-2
                - Landat 4-5 Thematic Mapper (TM)
                - Landsat 7 Enhanced Thematic Mapper Plus (ETM+)
                - Landsat 8-9 Operational Land Imager - Thermal Infrared Sensor
                  (OLI-TIRS)

        All assets (COGs, metadata files) must reside in the same
        directory/blob prefix/etc. as the MTL xml metadata file.

        \b
        Args:
            mtl (str): HREF to the source MTL metadata xml file
            output (str): Directory that will contain the STAC Item
            usgs_geometry (bool): Flag to use the geometry from a USGS STAC Item
                that resides in the same directory as the MTL xml file or can be
                queried from the USGS STAC API.
            legacy_l8 (bool): Flag to use the legacy method for creating a
                Landsat 8 STAC Item.
            antimeridian_strategy (str): Choice of 'normalize' or 'split' to
                either split the Item geometry on -180 longitude or normalize
                the Item geometry so all longitudes are either positive or
                negative. Has no effect if legacy_l8=True.
            level (str): Choice of 'level-1' or 'level-2'. This is not used
                and has no effect.
        """
        strategy = Strategy[antimeridian_strategy.upper()]
        item = create_stac_item(mtl_xml_href=mtl,
                                use_usgs_geometry=usgs_geometry,
                                legacy_l8=legacy_l8,
                                antimeridian_strategy=strategy)
        item.set_self_href(os.path.join(output, f'{item.id}.json'))
        item.save_object()

    @landsat.command(
        "create-collection",
        short_help="Creates a STAC Collection with contents defined by a list "
        " of metadata file hrefs in a text file.")
    @click.option("-f",
                  "--file_list",
                  required=True,
                  help="Text file of HREFs to Landsat scene XML MTL metadata "
                  "files.")
    @click.option("-o",
                  "--output",
                  required=True,
                  help="HREF of directory in which to write the collection.")
    @click.option("-i",
                  "--id",
                  type=click.Choice(['landsat-c2-l1', 'landsat-c2-l2'],
                                    case_sensitive=True),
                  required=True,
                  help="Landsat collection type. Choice of 'landsat-c2-l1' "
                  "'landsat-c2-l2'")
    @click.option("-u",
                  "--usgs_geometry",
                  is_flag=True,
                  help="Use USGS STAC Item geometry")
    @click.option("-a",
                  "--antimeridian_strategy",
                  type=click.Choice(["normalize", "split"],
                                    case_sensitive=False),
                  default="split",
                  show_default=True,
                  help="geometry strategy for antimeridian scenes")
    def create_collection_cmd(file_list: str, output: str, id: str,
                              usgs_geometry: bool,
                              antimeridian_strategy: str) -> None:
        """\b
        Creates a STAC Collection for Items defined by the hrefs in file_list.

        \b
        Args:
            file_list (str): Text file containing one href per line. The hrefs
                should point to XML MTL metadata files.
            output (str): Directory that will contain the collection.
            id (str): Choice of 'landsat-c2-l1' or 'landsat-c2-l2'.
            usgs_geometry (bool): Flag to use the geometry from a USGS STAC Item
                that resides in the same directory as the MTL xml file or can be
                queried from the USGS STAC API.
            antimeridian_strategy (str): Choice of 'normalize' or 'split' to
                either split the Item geometry on -180 longitude or normalize
                the Item geometry so all longitudes are either positive or
                negative.
        """
        strategy = Strategy[antimeridian_strategy.upper()]
        with open(file_list) as file:
            hrefs = [line.strip() for line in file.readlines()]

        collection = create_collection(id)
        collection.set_self_href(os.path.join(output, "collection.json"))
        collection.catalog_type = CatalogType.SELF_CONTAINED
        for href in hrefs:
            item = create_stac_item(href,
                                    use_usgs_geometry=usgs_geometry,
                                    legacy_l8=False,
                                    antimeridian_strategy=strategy)
            collection.add_item(item)
        collection.make_all_asset_hrefs_relative()
        collection.validate_all()
        collection.save()

    @landsat.command(
        "convert",
        short_help="Convert a USGS STAC 0.7.0 Item to an updated STAC Item")
    @click.option("-s",
                  "--stac",
                  required=True,
                  help="HREF to the source STAC file.")
    @click.option(
        "-p",
        "--enable-proj",
        is_flag=True,
        help="Enable the proj extension. Requires access to blue band.")
    @click.option("-d", "--dst", help="Output directory")
    def convert_cmd(stac: str, enable_proj: bool, dst: str) -> None:
        """Converts a USGS STAC 0.7.0 Item to an updated STAC Item.

        \b
        Args:
            stac (str): href to the source STAC file
            dst (str): Directory that will contain the STAC Item
            enable_proj: Flag to include the proj extension in the created STAC
                Item
        """
        in_item = Item.from_file(stac)
        item = transform_stac_to_stac(in_item, enable_proj=enable_proj)

        item_path = os.path.join(dst, '{}.json'.format(item.id))
        item.set_self_href(item_path)
        item.save_object()

    return landsat



import datetime
from typing import Any, Dict, Optional

import dateutil.parser
import rasterio
from pystac import Item, Link, MediaType, STACError
from pystac.extensions.eo import EOExtension
from pystac.extensions.projection import ProjectionExtension
from pystac.extensions.view import ViewExtension
from pystac_client import Client
from rasterio import RasterioIOError
from shapely.geometry import box, mapping, shape
from stactools.core.io import ReadHrefModifier
from stactools.core.utils import href_exists

from stactools.landsat.constants import (L8_EXTENSION_SCHEMA,
                                         OLD_L8_EXTENSION_SCHEMA, USGS_API,
                                         USGS_C2L1, USGS_C2L2_SR, Sensor)


def _parse_date(in_date: str) -> datetime.datetime:
    """
    Try to parse a date and return it as a datetime object with no timezone
    """
    parsed_date = dateutil.parser.parse(in_date)
    return parsed_date.replace(tzinfo=datetime.timezone.utc)


def transform_mtl_to_stac(metadata: dict) -> Item:
    """
    Handle USGS MTL as a dict and return a STAC item.

    NOT IMPLEMENTED

    Issues include:
        - There's no reference to UTM Zone or any other CRS info in the MTL
        - There's no absolute file path or reference to a URI to find data.
    """
    LANDSAT_METADATA = metadata["LANDSAT_METADATA_FILE"]
    product = LANDSAT_METADATA["PRODUCT_CONTENTS"]
    projection = LANDSAT_METADATA["PROJECTION_ATTRIBUTES"]
    image = LANDSAT_METADATA["IMAGE_ATTRIBUTES"]
    proessing_record = LANDSAT_METADATA["LEVEL2_PROCESSING_RECORD"]

    scene_id = product["LANDSAT_PRODUCT_ID"]

    xmin, xmax = float(projection["CORNER_LL_LON_PRODUCT"]), float(
        projection["CORNER_UR_LON_PRODUCT"])
    ymin, ymax = float(projection["CORNER_LL_LAT_PRODUCT"]), float(
        projection["CORNER_UR_LAT_PRODUCT"])
    geom = mapping(box(xmin, ymin, xmax, ymax))
    bounds = shape(geom).bounds

    # Like: "2020-01-01" for date and  "23:08:52.6773140Z" for time
    acquired_date = _parse_date(
        f"{image['DATE_ACQUIRED']}T{image['SCENE_CENTER_TIME']}")
    created = _parse_date(proessing_record["DATE_PRODUCT_GENERATED"])

    item = Item(id=scene_id,
                geometry=geom,
                bbox=bounds,
                datetime=acquired_date,
                properties={})

    # Common metadata
    item.common_metadata.created = created
    item.common_metadata.platform = image["SPACECRAFT_ID"]
    item.common_metadata.instruments = [
        i.lower() for i in image["SENSOR_ID"].split("_")
    ]

    # TODO: implement these three extensions
    EOExtension.add_to(item)
    ViewExtension.add_to(item)
    ProjectionExtension.add_to(item)

    return item


def transform_stac_to_stac(item: Item,
                           enable_proj: bool = True,
                           self_link: str = None,
                           source_link: str = None) -> Item:
    """
    Handle a 0.7.0 item and convert it to a 1.0.0.beta2 item.
    If `enable_proj` is true, the assets' geotiff files must be accessible.
    """
    # Clear hierarchical links
    item.set_parent(None)
    item.set_root(None)

    # Remove USGS extension and add back eo
    EOExtension.add_to(item)

    # Add and update links
    if self_link:
        item.links.append(Link(rel="self", target=self_link))
    if source_link:
        item.links.append(
            Link(rel="derived_from",
                 target=source_link,
                 media_type="application/json"))

    # Add some common fields
    item.common_metadata.constellation = "Landsat"

    # Handle view extension
    view = ViewExtension.ext(item, add_if_missing=True)
    if (item.properties.get("eo:off_nadir")
            or item.properties.get("eo:off_nadir") == 0):
        view.off_nadir = item.properties.pop("eo:off_nadir")
    elif (item.properties.get("view:off_nadir")
          or item.properties.get("view:off_nadir") == 0):
        view.off_nadir = item.properties.pop("view:off_nadir")
    else:
        STACError("eo:off_nadir or view:off_nadir is a required property")

    if enable_proj:
        # Enabled projection
        projection = ProjectionExtension.ext(item, add_if_missing=True)

        obtained_shape = None
        obtained_transform = None
        crs = None
        for asset in item.assets.values():
            if asset.media_type is not None and "geotiff" in asset.media_type:
                # retrieve shape, transform and crs from the first geotiff file among the assets
                if not obtained_shape:
                    try:
                        with rasterio.open(asset.href) as opened_asset:
                            obtained_shape = opened_asset.shape
                            obtained_transform = opened_asset.transform
                            crs = opened_asset.crs.to_epsg()
                            # Check to ensure that all information is present
                            if not obtained_shape or not obtained_transform or not crs:
                                raise STACError(
                                    f"Failed setting shape, transform and csr from {asset.href}"
                                )

                    except RasterioIOError as io_error:
                        raise STACError(
                            "Failed loading geotiff, so not handling proj fields"
                        ) from io_error

                asset_projection = ProjectionExtension.ext(asset)
                asset_projection.transform = obtained_transform
                asset_projection.shape = obtained_shape
                asset.media_type = MediaType.COG

        # Now we have the info, we can make the fields
        projection.epsg = crs

    # Remove .TIF from asset names
    item.assets = {
        name.replace(".TIF", ""): asset
        for name, asset in item.assets.items()
    }

    try:
        index = item.stac_extensions.index(OLD_L8_EXTENSION_SCHEMA)
        item.stac_extensions[index] = L8_EXTENSION_SCHEMA
    except ValueError:
        pass

    return item


def stac_api_to_stac(uri: str) -> Item:
    """
    Takes in a URI and uses that to feed the STAC transform
    """

    return transform_stac_to_stac(item=Item.from_file(uri),
                                  source_link=uri,
                                  enable_proj=False)


def get_usgs_geometry(
    base_href: str,
    sensor: Sensor,
    product_id: str,
    read_href_modifier: Optional[ReadHrefModifier] = None
) -> Optional[Dict[str, Any]]:
    """Attempts to get scene geometry from a USGS STAC Item.

    Args:
        base_href (str): Base href to a STAC storage location
        sensor (Sensor): Enum of MSS, TM, ETM, or OLI-TIRS
        product_id (str): Scene product id from mtl metadata
        read_href_modifier (Callable[[str], str]): An optional function to
            modify the storage href (e.g. to add a token to a url)
    Returns:
        Optional[Dict[str, Any]]: Either a GeoJSON geometry or None
    """
    # Check data storage first
    if sensor is Sensor.MSS:
        stac_href = f"{base_href}_stac.json"
    else:
        stac_href = f"{base_href}_SR_stac.json"

    if read_href_modifier is not None:
        stac_href = read_href_modifier(stac_href)

    if href_exists(stac_href):
        item = Item.from_file(stac_href)
    else:
        item = None

    # If not found, check the USGS STAC API
    if item is None:
        if sensor is Sensor.MSS:
            collection = USGS_C2L1
        else:
            collection = USGS_C2L2_SR
            product_id = f"{product_id}_SR"

        catalog = Client.open(USGS_API)
        search = catalog.search(collections=[collection], ids=[product_id])
        if search.matched() == 1:
            item = next(search.get_items())
        else:
            item = None

    if item is not None:
        return item.geometry
    else:
        return None



from collections import defaultdict
from datetime import datetime
from typing import Any, Dict, List, Optional

from pyproj import Geod
from pystac.utils import str_to_datetime
from stactools.core.io import ReadHrefModifier
from stactools.core.io.xml import XmlElement
from stactools.core.projection import transform_from_bbox
from stactools.core.utils import map_opt


class MTLError(Exception):
    pass


class MtlMetadata:
    """Parses a Collection 2 MTL XML file.

    References https://github.com/sat-utils/sat-stac-landsat/blob/f2263485043a827b4153aecc12f45a3d1363e9e2/satstac/landsat/main.py#L157
    """  # noqa

    def __init__(self,
                 root: XmlElement,
                 href: Optional[str] = None,
                 legacy_l8: bool = True):
        self._root = root
        self.href = href
        self.legacy_l8 = legacy_l8

    def _xml_error(self, item: str) -> MTLError:
        return MTLError(f"Cannot find {item} in MTL metadata" +
                        ("" if self.href is None else f" at {self.href}"))

    def _get_text(self, xpath: str) -> str:
        return self._root.find_text_or_throw(xpath, self._xml_error)

    def _get_float(self, xpath: str) -> float:
        return float(self._get_text(xpath))

    def _get_int(self, xpath: str) -> int:
        return int(self._get_text(xpath))

    @property
    def satellite_num(self) -> int:
        """Return the Landsat satellite number."""
        return int(self.product_id[2:4])

    @property
    def product_id(self) -> str:
        """Return the Landsat product ID."""
        return self._get_text("PRODUCT_CONTENTS/LANDSAT_PRODUCT_ID")

    @property
    def item_id(self) -> str:
        # Remove the processing date, as products IDs
        # that only vary by processing date represent the
        # same scene
        # See "Section 5 - Product Packaging" at
        # https://prd-wret.s3.us-west-2.amazonaws.com/assets/palladium/production/atoms/files/LSDS-1619_Landsat8-C2-L2-ScienceProductGuide-v2.pdf  # noqa

        # ID format: LXSS_LLLL_PPPRRR_YYYYMMDD_yyyymmdd_CX_TX
        # remove yyyymmdd
        id_parts = self.product_id.split('_')
        id = '_'.join(id_parts[:4] + id_parts[-2:])

        return id

    @property
    def scene_id(self) -> str:
        """"Return the Landsat scene ID."""
        return self._get_text("LEVEL1_PROCESSING_RECORD/LANDSAT_SCENE_ID")

    @property
    def processing_level(self) -> str:
        """Processing level. Determines product contents.

        Returns either 'L2SP' or 'L2SR', standing for
        'Level 2 Science Product' and 'Level 2 Surface Reflectance',
        respectively. L2SP has thermal + surface reflectance assets;
        L2SR only has surface reflectance.
        """
        return self._get_text("PRODUCT_CONTENTS/PROCESSING_LEVEL")

    @property
    def epsg(self) -> int:
        utm_zone = self._root.find_text('PROJECTION_ATTRIBUTES/UTM_ZONE')
        if utm_zone:
            if self.satellite_num == 8 and self.legacy_l8:
                # Keep current STAC Item content consistent for Landsat 8
                bbox = self.bbox
                utm_zone = self._get_text('PROJECTION_ATTRIBUTES/UTM_ZONE')
                center_lat = (bbox[1] + bbox[3]) / 2.0
                return int(f"{326 if center_lat > 0 else 327}{utm_zone}")
            else:
                # The projection transforms in the COGs provided by the USGS are
                # always for UTM North zones. The EPSG codes should therefore
                # be UTM north zones (326XX, where XX is the UTM zone number).
                # See: https://www.usgs.gov/faqs/why-do-landsat-scenes-southern-hemisphere-display-negative-utm-values  # noqa
                utm_zone = self._get_text('PROJECTION_ATTRIBUTES/UTM_ZONE')
                return int(f"326{utm_zone}")
        else:
            # Polar Stereographic
            # Based on Landsat 8-9 OLI/TIRS Collection 2 Level 1 Data Format Control Book,
            # should only ever be 71 or -71
            lat_ts = self._get_text('PROJECTION_ATTRIBUTES/TRUE_SCALE_LAT')
            if lat_ts == "-71.00000":
                # Antarctic
                return 3031
            elif lat_ts == "71.00000":
                # Arctic
                return 3995
            else:
                raise MTLError(
                    f'Unexpeced value for PROJECTION_ATTRIBUTES/TRUE_SCALE_LAT: {lat_ts} '
                )

    @property
    def bbox(self) -> List[float]:
        # Might be cleaner to just transform the proj bbox to WGS84.
        lons = [
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_UL_LON_PRODUCT"),
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_UR_LON_PRODUCT"),
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_LL_LON_PRODUCT"),
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_LR_LON_PRODUCT")
        ]

        lats = [
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_UL_LAT_PRODUCT"),
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_UR_LAT_PRODUCT"),
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_LL_LAT_PRODUCT"),
            self._get_float("PROJECTION_ATTRIBUTES/CORNER_LR_LAT_PRODUCT")
        ]
        geod = Geod(ellps="WGS84")
        offset = self.sr_gsd / 2
        _, _, bottom_distance = geod.inv(lons[2], lats[2], lons[3], lats[3])
        bottom_offset = offset * (lons[3] - lons[2]) / bottom_distance
        _, _, top_distance = geod.inv(lons[0], lats[0], lons[1], lats[1])
        top_offset = offset * (lons[1] - lons[0]) / top_distance
        _, _, lat_distance = geod.inv(lons[0], lats[0], lons[2], lats[2])
        lat_offset = offset * (lats[0] - lats[2]) / lat_distance
        return [
            min(lons) - bottom_offset,
            min(lats) - lat_offset,
            max(lons) + top_offset,
            max(lats) + lat_offset
        ]

    @property
    def proj_bbox(self) -> List[float]:
        # USGS metadata provide bounds at the center of the pixel, but
        # GDAL/rasterio transforms are to edge of pixel.
        # https://github.com/stac-utils/stactools/issues/117
        offset = self.sr_gsd / 2
        xs = [
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_UL_PROJECTION_X_PRODUCT") -
            offset,
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_UR_PROJECTION_X_PRODUCT") +
            offset,
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_LL_PROJECTION_X_PRODUCT") -
            offset,
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_LR_PROJECTION_X_PRODUCT") +
            offset
        ]

        ys = [
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_UL_PROJECTION_Y_PRODUCT") +
            offset,
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_UR_PROJECTION_Y_PRODUCT") +
            offset,
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_LL_PROJECTION_Y_PRODUCT") -
            offset,
            self._get_float(
                "PROJECTION_ATTRIBUTES/CORNER_LR_PROJECTION_Y_PRODUCT") -
            offset
        ]

        return [min(xs), min(ys), max(xs), max(ys)]

    @property
    def sr_shape(self) -> List[int]:
        """Shape for surface reflectance assets.

        Used for proj:shape. In [row, col] order"""
        return [
            self._get_int("PROJECTION_ATTRIBUTES/REFLECTIVE_LINES"),
            self._get_int("PROJECTION_ATTRIBUTES/REFLECTIVE_SAMPLES")
        ]

    @property
    def thermal_shape(self) -> Optional[List[int]]:
        """Shape for thermal bands.

        None if thermal bands not present.
        Used for proj:shape. In [row, col] order"""
        rows = map_opt(
            int, self._root.find_text("PROJECTION_ATTRIBUTES/THERMAL_LINES"))
        cols = map_opt(
            int, self._root.find_text("PROJECTION_ATTRIBUTES/THERMAL_SAMPLES"))

        if rows is not None and cols is not None:
            return [rows, cols]
        else:
            return None

    @property
    def sr_transform(self) -> List[float]:
        return transform_from_bbox(self.proj_bbox, self.sr_shape)

    @property
    def thermal_transform(self) -> Optional[List[float]]:
        return map_opt(
            lambda shape: transform_from_bbox(self.proj_bbox, shape),
            self.thermal_shape)

    @property
    def sr_gsd(self) -> float:
        return self._get_float(
            "LEVEL1_PROJECTION_PARAMETERS/GRID_CELL_SIZE_REFLECTIVE")

    @property
    def thermal_gsd(self) -> Optional[float]:
        return map_opt(
            float,
            self._root.find_text(
                'LEVEL1_PROJECTION_PARAMETERS/GRID_CELL_SIZE_THERMAL'))

    @property
    def scene_datetime(self) -> datetime:
        date = self._get_text("IMAGE_ATTRIBUTES/DATE_ACQUIRED")
        time = self._get_text("IMAGE_ATTRIBUTES/SCENE_CENTER_TIME")

        return str_to_datetime(f"{date} {time}")

    @property
    def cloud_cover(self) -> float:
        return self._get_float("IMAGE_ATTRIBUTES/CLOUD_COVER")

    @property
    def sun_azimuth(self) -> float:
        """Returns the sun azimuth in STAC form.

        Converts from Landsat metadata form (-180 to 180 from north, west being
        negative) to STAC form (0 to 360 clockwise from north).

        Returns:
            float: Sun azimuth, 0 to 360 clockwise from north.
        """
        azimuth = self._get_float("IMAGE_ATTRIBUTES/SUN_AZIMUTH")
        if azimuth < 0.0:
            azimuth += 360
        return azimuth

    @property
    def sun_elevation(self) -> float:
        return self._get_float("IMAGE_ATTRIBUTES/SUN_ELEVATION")

    @property
    def off_nadir(self) -> Optional[float]:
        if self.satellite_num == 8 and self.legacy_l8:
            # Keep current STAC Item content consistent for Landsat 8
            if self._get_text("IMAGE_ATTRIBUTES/NADIR_OFFNADIR") == "NADIR":
                return 0
            else:
                return None
        else:
            # NADIR_OFFNADIR and ROLL_ANGLE xml entries do not exist prior to
            # landsat 8. Therefore, we perform a soft check for NADIR_OFFNADIR.
            # If exists and is equal to "OFFNADIR", then a non-zero ROLL_ANGLE
            # exists. We force this ROLL_ANGLE to be positive to conform with
            # the stac View Geometry extension. We return 0 otherwise since
            # off-nadir views are only an option on Landsat 8-9.
            if self._root.find_text(
                    "IMAGE_ATTRIBUTES/NADIR_OFFNADIR") == "OFFNADIR":
                return abs(self._get_float("IMAGE_ATTRIBUTES/ROLL_ANGLE"))
            else:
                return 0

    @property
    def wrs_path(self) -> str:
        return self._get_text("IMAGE_ATTRIBUTES/WRS_PATH").zfill(3)

    @property
    def wrs_row(self) -> str:
        return self._get_text("IMAGE_ATTRIBUTES/WRS_ROW").zfill(3)

    @property
    def landsat_metadata(self) -> Dict[str, Any]:
        landsat_meta = {
            "landsat:cloud_cover_land":
            self._get_float("IMAGE_ATTRIBUTES/CLOUD_COVER_LAND"),
            "landsat:wrs_type":
            self._get_text("IMAGE_ATTRIBUTES/WRS_TYPE"),
            "landsat:wrs_path":
            self.wrs_path,
            "landsat:wrs_row":
            self.wrs_row,
            "landsat:collection_category":
            self._get_text("PRODUCT_CONTENTS/COLLECTION_CATEGORY"),
            "landsat:collection_number":
            self._get_text("PRODUCT_CONTENTS/COLLECTION_NUMBER"),
            "landsat:correction":
            self.processing_level,
            "landsat:scene_id":
            self.scene_id
        }
        if self.satellite_num == 8 and self.legacy_l8:
            landsat_meta["landsat:processing_level"] = landsat_meta.pop(
                "landsat:correction")
        return landsat_meta

    @property
    def level1_radiance(self) -> Dict[str, Any]:
        """Gets the scale (mult) and offset (add) values for generating TOA
        radiance from Level-1 DNs.

        This is relevant to MSS data, which is only processed to Level-1.

        Returns:
            Dict[str, Any]: Dict of scale and offset dicts, keyed by band
                number.
        """
        node = self._root.find_or_throw("LEVEL1_RADIOMETRIC_RESCALING",
                                        self._xml_error)
        mult_add: Dict[str, Any] = defaultdict(dict)
        for item in node.element:
            if item.tag.startswith("RADIANCE_MULT_BAND"):
                band = f'B{item.tag.split("_")[-1]}'
                mult_add[band]["mult"] = float(str(item.text))
            elif item.tag.startswith("RADIANCE_ADD_BAND"):
                band = f'B{item.tag.split("_")[-1]}'
                mult_add[band]["add"] = float(str(item.text))
        return mult_add

    @classmethod
    def from_file(cls,
                  href: str,
                  read_href_modifier: Optional[ReadHrefModifier] = None,
                  legacy_l8: bool = True) -> "MtlMetadata":
        return cls(XmlElement.from_file(href, read_href_modifier),
                   href=href,
                   legacy_l8=legacy_l8)



import logging
from datetime import datetime, timezone
from typing import Optional

from pystac import Collection, Item, Link, MediaType
from pystac.extensions.eo import Band, EOExtension
from pystac.extensions.item_assets import ItemAssetsExtension
from pystac.extensions.projection import ProjectionExtension
from pystac.extensions.raster import RasterBand, RasterExtension
from pystac.extensions.scientific import ScientificExtension
from pystac.extensions.view import ViewExtension
from shapely.geometry import box, mapping
from stactools.core.io import ReadHrefModifier
from stactools.core.utils import antimeridian
from stactools.core.utils.antimeridian import Strategy

from stactools.landsat.ang_metadata import AngMetadata
from stactools.landsat.assets import (ANG_ASSET_DEF, COMMON_ASSET_DEFS,
                                      SR_ASSET_DEFS, THERMAL_ASSET_DEFS)
from stactools.landsat.constants import (COLLECTION_IDS, L8_EXTENSION_SCHEMA,
                                         L8_INSTRUMENTS, L8_ITEM_DESCRIPTION,
                                         L8_PLATFORM, LANDSAT_EXTENSION_SCHEMA,
                                         SENSORS, USGS_API, USGS_BROWSER_C2,
                                         USGS_C2L1, USGS_C2L2_SR, USGS_C2L2_ST,
                                         Sensor)
from stactools.landsat.fragments import CollectionFragments, Fragments
from stactools.landsat.mtl_metadata import MtlMetadata
from stactools.landsat.utils import get_usgs_geometry

logger = logging.getLogger(__name__)


def create_stac_item(
        mtl_xml_href: str,
        legacy_l8: bool = True,
        use_usgs_geometry: bool = False,
        antimeridian_strategy: Strategy = Strategy.SPLIT,
        read_href_modifier: Optional[ReadHrefModifier] = None) -> Item:
    """Creates a STAC Item for Landsat 1-5 Collection 2 Level-1 or Landsat
    4-5, 7-9 Collection 2 Level-2 scene data.

    Args:
        mtl_xml_href (str): An href to an MTL XML metadata file.
        legacy_l8 (bool): Use the legacy (old) method for a Landsat 8 STAC Item.
        use_usgs_geometry (bool): Use the geometry from a USGS STAC file that is
            stored alongside the XML metadata file or pulled from the USGS STAC
            API.
        antimeridian_strategy (Antimeridian): Either split on -180 or
            normalize geometries so all longitudes are either positive or
            negative. Has no effect if legacy_l8=True.
        read_href_modifier (Callable[[str], str]): An optional function to
            modify the MTL and USGS STAC hrefs (e.g. to add a token to a url).
    Returns:
        pystac.Item: A STAC Item representing the Landsat scene.
    """
    base_href = '_'.join(mtl_xml_href.split('_')[:-1])

    mtl_metadata = MtlMetadata.from_file(mtl_xml_href,
                                         read_href_modifier,
                                         legacy_l8=legacy_l8)

    sensor = Sensor(mtl_metadata.item_id[1])
    satellite = int(mtl_metadata.item_id[2:4])
    level = int(mtl_metadata.item_id[6])
    correction = mtl_metadata.item_id[7:9]

    if use_usgs_geometry:
        geometry = get_usgs_geometry(base_href, sensor,
                                     mtl_metadata.product_id,
                                     read_href_modifier)
    else:
        geometry = None

    if geometry is None:
        if sensor is Sensor.OLI_TIRS:
            ang_href = ANG_ASSET_DEF.get_href(base_href)
            ang_metadata = AngMetadata.from_file(ang_href, read_href_modifier)
            geometry = ang_metadata.get_scene_geometry(mtl_metadata.bbox)
        else:
            geometry = mapping(box(*mtl_metadata.bbox))
            logger.warning(
                f"Using bbox for geometry for {mtl_metadata.product_id}.")

    item = Item(id=mtl_metadata.item_id,
                bbox=mtl_metadata.bbox,
                geometry=geometry,
                datetime=mtl_metadata.scene_datetime,
                properties={})

    if satellite == 8 and legacy_l8:
        item.common_metadata.platform = L8_PLATFORM
        item.common_metadata.instruments = L8_INSTRUMENTS
        item.common_metadata.description = L8_ITEM_DESCRIPTION

        # eo
        eo_item = EOExtension.ext(item, add_if_missing=True)
        eo_item.cloud_cover = mtl_metadata.cloud_cover

        # view
        view = ViewExtension.ext(item, add_if_missing=True)
        view.off_nadir = mtl_metadata.off_nadir
        view.sun_elevation = mtl_metadata.sun_elevation
        view.sun_azimuth = mtl_metadata.sun_azimuth

        # projection
        projection = ProjectionExtension.ext(item, add_if_missing=True)
        projection.epsg = mtl_metadata.epsg
        projection.bbox = mtl_metadata.proj_bbox

        # landsat8
        item.stac_extensions.append(L8_EXTENSION_SCHEMA)
        item.properties.update(**mtl_metadata.landsat_metadata)

        # -- Add assets

        # Add common assets
        for asset_definition in COMMON_ASSET_DEFS:
            asset_definition.add_asset(item, mtl_metadata, base_href)

        # Add SR assets
        for asset_definition in SR_ASSET_DEFS:
            asset_definition.add_asset(item, mtl_metadata, base_href)

        # Add thermal assets, if this is a L2SP product
        if mtl_metadata.processing_level == 'L2SP':
            for asset_definition in THERMAL_ASSET_DEFS:
                asset_definition.add_asset(item, mtl_metadata, base_href)

        # -- Add links
        # NOTE: This link is incorrect. Leads to a dead "page". The last
        # component needs to have the processing time in it (replace item_id
        # with product_id to fix).
        usgs_item_page = (f"{USGS_BROWSER_C2}/level-2/standard/oli-tirs"
                          f"/{mtl_metadata.scene_datetime.year}"
                          f"/{mtl_metadata.wrs_path}/{mtl_metadata.wrs_row}"
                          f"/{mtl_metadata.item_id}")

        item.add_link(
            Link(rel="alternate",
                 target=usgs_item_page,
                 title="USGS stac-browser page",
                 media_type="text/html"))

    else:
        antimeridian.fix_item(item, antimeridian_strategy)

        item.common_metadata.platform = f"landsat-{satellite}"
        item.common_metadata.instruments = SENSORS[sensor.name]["instruments"]
        item.common_metadata.created = datetime.now(tz=timezone.utc)
        item.common_metadata.gsd = SENSORS[sensor.name]["reflective_gsd"]
        item.common_metadata.description = f"Landsat Collection 2 Level-{level}"

        fragments = Fragments(sensor, satellite, base_href,
                              mtl_metadata.level1_radiance)

        # Common assets
        assets = fragments.common_assets()
        for key, asset in assets.items():
            # MSS data does not have an angle file
            if sensor is Sensor.MSS and key.startswith("ang"):
                continue
            # MTL files are specific to the processing level
            if key.startswith("mtl"):
                asset.description = asset.description.replace(
                    "Level-X", f"Level-{level}")
            item.add_asset(key, asset)

        # Optical assets
        assets = fragments.sr_assets()
        eo_bands = fragments.sr_eo_bands()
        raster_bands = fragments.sr_raster_bands()
        for key, asset in assets.items():
            item.add_asset(key, asset)
            eo_band = eo_bands.get(key, None)
            if eo_band is not None:
                optical_eo = EOExtension.ext(asset, add_if_missing=True)
                optical_eo.bands = [Band.create(**eo_band)]
            raster_band = raster_bands.get(key, None)
            if raster_band is not None:
                optical_raster = RasterExtension.ext(asset,
                                                     add_if_missing=True)
                optical_raster.bands = [RasterBand.create(**raster_band)]

        # Thermal assets (only exist if optical exist)
        if mtl_metadata.processing_level == 'L2SP':
            assets = fragments.st_assets()
            eo_bands = fragments.st_eo_bands()
            raster_bands = fragments.st_raster_bands()
            for key, asset in assets.items():
                item.add_asset(key, asset)
                eo_band = eo_bands.get(key, None)
                if eo_band is not None:
                    thermal_eo = EOExtension.ext(asset, add_if_missing=True)
                    thermal_eo.bands = [Band.create(**eo_band)]
                raster_band = raster_bands.get(key, None)
                if raster_band is not None:
                    thermal_raster = RasterExtension.ext(asset,
                                                         add_if_missing=True)
                    thermal_raster.bands = [RasterBand.create(**raster_band)]
                if key.startswith("lwir"):
                    asset.common_metadata.gsd = SENSORS[
                        sensor.name]["thermal_gsd"]

        eo_item = EOExtension.ext(item, add_if_missing=True)
        eo_item.cloud_cover = mtl_metadata.cloud_cover

        view = ViewExtension.ext(item, add_if_missing=True)
        view.off_nadir = mtl_metadata.off_nadir
        view.sun_elevation = mtl_metadata.sun_elevation
        view.sun_azimuth = mtl_metadata.sun_azimuth

        projection = ProjectionExtension.ext(item, add_if_missing=True)
        projection.epsg = mtl_metadata.epsg
        projection.shape = mtl_metadata.sr_shape
        projection.transform = mtl_metadata.sr_transform

        item.stac_extensions.append(LANDSAT_EXTENSION_SCHEMA)
        item.properties.update(**mtl_metadata.landsat_metadata)

        scientific = ScientificExtension.ext(item, add_if_missing=True)
        scientific.doi = SENSORS[sensor.name]["doi"]
        doi_link = item.get_single_link("cite-as")
        doi_link.title = SENSORS[sensor.name]["doi_title"]  # type: ignore

        via_links = []
        if level == 1:
            via_links.append(
                f"{USGS_API}/collections/{USGS_C2L1}/items/{mtl_metadata.product_id}"
            )
        elif level == 2 and correction == "SP":
            via_links.append(
                f"{USGS_API}/collections/{USGS_C2L2_SR}/items/{mtl_metadata.product_id}_SR"
            )
            via_links.append(
                f"{USGS_API}/collections/{USGS_C2L2_ST}/items/{mtl_metadata.product_id}_ST"
            )
        elif level == 2 and correction == "SR":
            via_links.append(
                f"{USGS_API}/collections/{USGS_C2L2_SR}/items/{mtl_metadata.product_id}_SR"
            )
        for via_link in via_links:
            item.add_link(
                Link(rel="via",
                     target=via_link,
                     title="USGS STAC Item",
                     media_type=MediaType.JSON))

    return item


def create_collection(collection_id: str) -> Collection:
    """Creates a STAC Collection for Landsat Collection 2 Level-1 or Level-2
    data.

    Args:
        collection_id (str): ID of the STAC Collection. Must be one of
            "landsat-c2-l1" or "landsat-c2-l2".
    Returns:
        Collection: The created STAC Collection.
    """
    if collection_id not in COLLECTION_IDS:
        raise ValueError(f"Invalid collection id: {collection_id}")

    fragment = CollectionFragments(collection_id).collection()

    collection = Collection(id=collection_id,
                            title=fragment["title"],
                            description=fragment["description"],
                            license=fragment["license"],
                            keywords=fragment["keywords"],
                            providers=fragment["providers"],
                            extent=fragment["extent"],
                            summaries=fragment["summaries"])
    collection.add_links(fragment["links"])

    item_assets = ItemAssetsExtension(collection)
    item_assets.item_assets = fragment["item_assets"]

    ItemAssetsExtension.add_to(collection)
    ViewExtension.add_to(collection)
    ScientificExtension.add_to(collection)
    RasterExtension.add_to(collection)
    EOExtension.add_to(collection)

    return collection



import json
from typing import Any, Dict, Optional

import pkg_resources
from pystac import Asset, Extent, Link, MediaType, Provider, Summaries
from pystac.extensions.item_assets import AssetDefinition
from pystac.utils import make_absolute_href

from stactools.landsat.constants import Sensor


class CollectionFragments:
    """Class for accessing collection data."""

    def __init__(self, collection_id: str):
        """Initialize a new group of fragments for the provided Sensor."""
        self._id = collection_id

    def collection(self) -> Dict[str, Any]:
        """Loads the collection.json for the given collection id.

        Converts some elements to pystac object.

        Returns:
            Dict[str, Any]: Dict from parsed JSON with some converted fields.
        """
        data = self._load()
        data["extent"] = Extent.from_dict(data["extent"])
        data["providers"] = [
            Provider.from_dict(provider) for provider in data["providers"]
        ]
        data["links"] = [Link.from_dict(link) for link in data["links"]]
        data["summaries"] = Summaries(data["summaries"])

        item_assets = {}
        for key, asset_dict in data["item_assets"].items():
            media_type = asset_dict.get("type")
            asset_dict["type"] = MediaType[media_type]
            item_assets[key] = AssetDefinition(asset_dict)
        data["item_assets"] = item_assets

        return data

    def _load(self) -> Any:
        try:
            with pkg_resources.resource_stream(
                    "stactools.landsat.fragments",
                    f"collections/{self._id}.json") as stream:
                return json.load(stream)
        except FileNotFoundError as e:
            raise e


class Fragments:
    """Class for accessing asset and extension data."""

    def __init__(self, sensor: Sensor, satellite: int, base_href: str,
                 level1_radiance: Dict[str, Dict[str, float]]):
        """Initialize a new group of fragments for the provided Sensor."""
        self._sensor = sensor
        self._satellite = satellite
        self.base_href = base_href
        self.level1_radiance = level1_radiance

    def common_assets(self) -> Dict[str, Any]:
        """Loads common-assets.json.

        Converts the loaded dicts to STAC Assets.

        Returns:
            Dict[str, Asset]: Dict of Assets keys and Assets.
        """
        asset_dicts = self._load("common-assets.json", "common")
        assets = self._convert_assets(asset_dicts)
        return assets

    def sr_assets(self) -> Dict[str, Any]:
        """Loads the sr-assets.json for the given sensor.

        If MSS, updates the band numbers depending on the given satellite number
        (satellites 1-3 use bands 4-7, satellites 4-5 use bands 1-4). Converts
        the loaded dicts to STAC Assets.

        Returns:
            Dict[str, Asset]: Dict of Assets keys and Assets.
        """
        asset_dicts = self._load("sr-assets.json")
        if self._satellite < 4:
            asset_dicts = self._update_mss_num(asset_dicts)
        assets = self._convert_assets(asset_dicts)
        return assets

    def sr_eo_bands(self) -> Dict[str, Any]:
        """Loads the sr-eo-bands.json for the given sensor.

        If MSS, updates the band numbers depending on the given satellite number
        (satellites 1-3 use bands 4-7, satellites 4-5 use bands 1-4).

        Returns:
            Dict[str, Dict]: Dict of Assets keys and EO Extension dicts.
        """
        eo = self._load("sr-eo-bands.json")
        if self._satellite < 4:
            eo = self._update_mss_num(eo)
        return eo

    def sr_raster_bands(self) -> Dict[str, Any]:
        """Loads the sr-raster-bands.json for the given sensor.

        If MSS, updates the band numbers depending on the given satellite number
        (satellites 1-3 use bands 4-7, satellites 4-5 use bands 1-4). If MSS,
        adds scale and offset informationt.

        Returns:
            Dict[str, Dict]: Dict of Assets keys and Raster Extension dicts.
        """
        raster = self._load("sr-raster-bands.json")
        if self._satellite < 4:
            raster = self._update_mss_num(raster)
        if self._sensor is Sensor.MSS:
            raster = self._update_mss_raster(raster)
        return raster

    def st_assets(self) -> Dict[str, Any]:
        """Loads the st-assets.json for the given sensor.

        Converts the loaded dicts to STAC Assets.

        Returns:
            Dict[str, Asset]: Dict of Assets keys and Assets.
        """
        asset_dicts = self._load("st-assets.json")
        assets = self._convert_assets(asset_dicts)
        return assets

    def st_eo_bands(self) -> Dict[str, Any]:
        """Loads the st-eo-bands.json for the given sensor.

        Returns:
            Dict[str, Dict]: Dict of Assets keys and EO Extension dicts.
        """
        eo = self._load("st-eo-bands.json")
        return eo

    def st_raster_bands(self) -> Dict[str, Any]:
        """Loads the st-raster-bands.json for the given sensor.

        Returns:
            Dict[str, Dict]: Dict of Assets keys and Raster Extension dicts.
        """
        raster = self._load("st-raster-bands.json")
        return raster

    def _update_mss_num(self, mss_dict: Dict[str, Any]) -> Dict[str, Any]:
        mss_str = json.dumps(mss_dict)
        mss_str = mss_str.replace("B4", "B7")
        mss_str = mss_str.replace("B3", "B6")
        mss_str = mss_str.replace("B2", "B5")
        mss_str = mss_str.replace("B1", "B4")
        mss_str = mss_str.replace("4-5", "1-3")
        return json.loads(mss_str)

    def _update_mss_raster(self, mss_raster_dict: Dict[str,
                                                       Any]) -> Dict[str, Any]:
        for key, value in mss_raster_dict.items():
            rad_key = value.pop("temp_name", None)
            if rad_key:
                mss_raster_dict[key]["scale"] = self.level1_radiance[rad_key][
                    "mult"]
                mss_raster_dict[key]["offset"] = self.level1_radiance[rad_key][
                    "add"]

        return mss_raster_dict

    def _convert_assets(self, asset_dicts: Dict[str, Any]) -> Dict[str, Asset]:
        assets = {}
        for key, asset_dict in asset_dicts.items():
            media_type = asset_dict.pop("type", None)
            if media_type is not None:
                asset_dict["type"] = MediaType[media_type]
            else:
                asset_dict["type"] = MediaType.COG

            href_suffix = asset_dict.pop('href_suffix', None)
            if href_suffix is not None:
                href = f"{self.base_href}_{href_suffix}"
            else:
                href = f"{self.base_href}_{key.upper()}.TIF"
            asset_dict["href"] = make_absolute_href(href)
            assets[key] = Asset.from_dict(asset_dict)

        return assets

    def _load(self, file_name: str, dir_name: Optional[str] = None) -> Any:
        if dir_name is None:
            dir_name = self._sensor.name.lower()
        try:
            with pkg_resources.resource_stream(
                    "stactools.landsat.fragments",
                    f"{dir_name}/{file_name}") as stream:
                return json.load(stream)
        except FileNotFoundError as e:
            raise e



from enum import Enum
from typing import Any, Dict

from pystac.extensions.eo import Band


class Sensor(Enum):
    MSS = "M"
    TM = "T"
    ETM = "E"
    OLI_TIRS = "C"


LANDSAT_EXTENSION_SCHEMA = "https://landsat.usgs.gov/stac/landsat-extension/v1.1.1/schema.json"
USGS_API = "https://landsatlook.usgs.gov/stac-server"
USGS_BROWSER_C2 = "https://landsatlook.usgs.gov/stac-browser/collection02"
USGS_C2L1 = "landsat-c2l1"
USGS_C2L2_SR = "landsat-c2l2-sr"
USGS_C2L2_ST = "landsat-c2l2-st"
COLLECTION_IDS = ["landsat-c2-l1", "landsat-c2-l2"]

SENSORS: Dict[str, Any] = {
    "MSS": {
        "instruments": ["mss"],
        "doi": "10.5066/P9AF14YV",
        "doi_title": "Landsat 1-5 MSS Collection 2 Level-1",
        "reflective_gsd": 79
    },
    "TM": {
        "instruments": ["tm"],
        "doi": "10.5066/P9IAXOVV",
        "doi_title": "Landsat 4-5 TM Collection 2 Level-2",
        "reflective_gsd": 30,
        "thermal_gsd": 120
    },
    "ETM": {
        "instruments": ["etm+"],
        "doi": "10.5066/P9C7I13B",
        "doi_title": "Landsat 7 ETM+ Collection 2 Level-2",
        "reflective_gsd": 30,
        "thermal_gsd": 60
    },
    "OLI_TIRS": {
        "instruments": ["oli", "tirs"],
        "doi": "10.5066/P9OGBGM6",
        "doi_title": "Landsat 8-9 OLI/TIRS Collection 2 Level-2",
        "reflective_gsd": 30,
        "thermal_gsd": 100
    }
}

L8_PLATFORM = "landsat-8"
L8_INSTRUMENTS = ["oli", "tirs"]

OLD_L8_EXTENSION_SCHEMA = "https://landsat.usgs.gov/stac/landsat-extension/schema.json"
L8_EXTENSION_SCHEMA = "https://landsat.usgs.gov/stac/landsat-extension/v1.1.0/schema.json"
L8_ITEM_DESCRIPTION = "Landsat Collection 2 Level-2 Surface Reflectance Product"

L8_SR_BANDS = {
    "SR_B1":
    Band({
        "name": "SR_B1",
        "common_name": "coastal",
        "gsd": 30,
        "center_wavelength": 0.44,
        "full_width_half_max": 0.02
    }),
    "SR_B2":
    Band({
        "name": "SR_B2",
        "common_name": "blue",
        "gsd": 30,
        "center_wavelength": 0.48,
        "full_width_half_max": 0.06
    }),
    "SR_B3":
    Band({
        "name": "SR_B3",
        "common_name": "green",
        "gsd": 30,
        "center_wavelength": 0.56,
        "full_width_half_max": 0.06
    }),
    "SR_B4":
    Band({
        "name": "SR_B4",
        "common_name": "red",
        "gsd": 30,
        "center_wavelength": 0.65,
        "full_width_half_max": 0.04
    }),
    "SR_B5":
    Band({
        "name": "SR_B5",
        "common_name": "nir08",
        "gsd": 30,
        "center_wavelength": 0.86,
        "full_width_half_max": 0.03
    }),
    "SR_B6":
    Band({
        "name": "SR_B6",
        "common_name": "swir16",
        "gsd": 30,
        "center_wavelength": 1.6,
        "full_width_half_max": 0.08
    }),
    "SR_B7":
    Band({
        "name": "SR_B7",
        "common_name": "swir22",
        "gsd": 30,
        "center_wavelength": 2.2,
        "full_width_half_max": 0.2
    })
}

L8_SP_BANDS = {
    # L2SP only bands

    #  ST_B10 Note:
    # Changed common_name from UGSG STAC - should be lwir11 based on wavelength
    # Also, resolution at sensor is 100m, even though the raster is 30m pixel width/height.
    "ST_B10":
    Band({
        "name": "ST_B10",
        "common_name": "lwir11",
        "gsd": 100.0,
        "center_wavelength": 10.9,
        "full_width_half_max": 0.8
    }),
    "ST_ATRAN":
    Band({
        "name": "ST_ATRAN",
        "description": "atmospheric transmission",
        "gsd": 30
    }),
    "ST_CDIST":
    Band({
        "name": "ST_CDIST",
        "description": "distance to nearest cloud",
        "gsd": 30
    }),
    "ST_DRAD":
    Band({
        "name": "ST_DRAD",
        "description": "downwelled radiance",
        "gsd": 30
    }),
    "ST_URAD":
    Band({
        "name": "ST_URAD",
        "description": "upwelled radiance",
        "gsd": 30
    }),
    "ST_TRAD":
    Band({
        "name": "ST_TRAD",
        "description": "thermal radiance",
        "gsd": 30
    }),
    "ST_EMIS":
    Band({
        "name": "ST_EMIS",
        "description": "emissivity",
        "gsd": 30
    }),
    "ST_EMSD":
    Band({
        "name": "ST_EMSD",
        "description": "emissivity standard deviation",
        "gsd": 30
    })
}
